@@ -1,18 +1,27 @@
+import validators
 from django.conf import settings
-from django.contrib.auth.models import AnonymousUser, User
+from django.contrib.auth import get_user_model
 from django.db import models
 from django.db.models.base import ModelBase
-from django.urls import get_resolver
+from django.db.models.signals import post_save
+from django.dispatch import receiver
+from django.urls import reverse_lazy, get_resolver
+from django.utils.datastructures import MultiValueDictKeyError
 from django.utils.decorators import classonlymethod
-from guardian.shortcuts import get_perms
 
-User._meta.rdf_type = "foaf:user"
+from djangoldp.fields import LDPUrlField
+from djangoldp.permissions import LDPPermissions
 
 
 class Model(models.Model):
+    urlid = LDPUrlField(blank=True, null=True, unique=True)
+
+    def __init__(self, *args, **kwargs):
+        super(Model, self).__init__(*args, **kwargs)
 
     @classmethod
     def get_view_set(cls):
+        '''returns the view_set defined in the model Meta or the LDPViewSet class'''
         view_set = getattr(cls._meta, 'view_set', getattr(cls.Meta, 'view_set', None))
         if view_set is None:
             from djangoldp.views import LDPViewSet
@@ -21,21 +30,36 @@ class Model(models.Model):
 
     @classmethod
     def get_container_path(cls):
+        '''returns the url path which is used to access actions on this model (e.g. /users/)'''
         path = getattr(cls._meta, 'container_path', getattr(cls.Meta, 'container_path', None))
         if path is None:
             path = "{}s".format(cls._meta.object_name.lower())
 
-        return path
+        return cls.__clean_path(path)
 
     def get_absolute_url(self):
-        return Model.resource_id(self)
+        return Model.absolute_url(self)
+
+    @classonlymethod
+    def absolute_url(cls, instance_or_model):
+        if isinstance(instance_or_model, ModelBase) or instance_or_model.urlid is None or instance_or_model.urlid == '':
+            return '{}{}'.format(settings.SITE_URL, Model.resource(instance_or_model))
+        else:
+            return instance_or_model.urlid
 
     def get_container_id(self):
         return Model.container_id(self)
 
+    @classonlymethod
+    def resource(cls, instance_or_model):
+        if isinstance(instance_or_model, ModelBase):
+            return cls.container_id(instance_or_model)
+        else:
+            return cls.resource_id(instance_or_model)
+
     @classonlymethod
     def resource_id(cls, instance):
-        r_id = "{}{}".format(cls.container_id(instance), getattr(instance, cls.slug_field(instance)))
+        r_id = "{}{}".format(cls.container_id(instance), getattr(instance, cls.slug_field(instance), ""))
         return cls.__clean_path(r_id)
 
     @classonlymethod
@@ -45,7 +69,10 @@ class Model(models.Model):
         else:
             object_name = instance_or_model._meta.object_name.lower()
         view_name = '{}-detail'.format(object_name)
-        slug_field = '/{}'.format(get_resolver().reverse_dict[view_name][0][0][1][0])
+        try:
+            slug_field = '/{}'.format(get_resolver().reverse_dict[view_name][0][0][1][0])
+        except MultiValueDictKeyError:
+            slug_field = Model.get_meta(instance_or_model, 'lookup_field', 'pk')
         if slug_field.startswith('/'):
             slug_field = slug_field[1:]
         return slug_field
@@ -65,8 +92,7 @@ class Model(models.Model):
     class Meta:
         default_permissions = ('add', 'change', 'delete', 'view', 'control')
         abstract = True
-        depth = 1
-        many_depth = 0
+        depth = 0
 
     @classonlymethod
     def resolve_id(cls, id):
@@ -74,6 +100,12 @@ class Model(models.Model):
         view, args, kwargs = get_resolver().resolve(id)
         return view.initkwargs['model'].objects.get(**kwargs)
 
+    @classonlymethod
+    def resolve_parent(cls, path):
+        split = path.strip('/').split('/')
+        parent_path = "/".join(split[0:len(split) - 1])
+        return Model.resolve_id(parent_path)
+
     @classonlymethod
     def resolve_container(cls, path):
         path = cls.__clean_path(path)
@@ -82,6 +114,8 @@ class Model(models.Model):
 
     @classonlymethod
     def resolve(cls, path):
+        if settings.BASE_URL in path:
+            path = path[len(settings.BASE_URL):]
         container = cls.resolve_container(path)
         try:
             resolve_id = cls.resolve_id(path)
@@ -91,6 +125,7 @@ class Model(models.Model):
 
     @classonlymethod
     def __clean_path(cls, path):
+        '''ensures path is Django-friendly'''
         if not path.startswith("/"):
             path = "/{}".format(path)
         if not path.endswith("/"):
@@ -99,10 +134,12 @@ class Model(models.Model):
 
     @classonlymethod
     def get_permission_classes(cls, related_model, default_permissions_classes):
+        '''returns the permission_classes set in the models Meta class'''
         return cls.get_meta(related_model, 'permission_classes', default_permissions_classes)
 
     @classonlymethod
     def get_meta(cls, model_class, meta_name, default=None):
+        '''returns the models Meta class'''
         if hasattr(model_class, 'Meta'):
             meta = getattr(model_class.Meta, meta_name, default)
         else:
@@ -112,25 +149,52 @@ class Model(models.Model):
     @staticmethod
     def get_permissions(obj_or_model, user_or_group, filter):
         permissions = filter
-        for permission_class in Model.get_permission_classes(obj_or_model, []):
+        for permission_class in Model.get_permission_classes(obj_or_model, [LDPPermissions]):
             permissions = permission_class().filter_user_perms(user_or_group, obj_or_model, permissions)
-
-        if not isinstance(user_or_group, AnonymousUser):
-            permissions += get_perms(user_or_group, obj_or_model)
         return [{'mode': {'@type': name.split('_')[0]}} for name in permissions]
 
+    @classmethod
+    def is_external(cls, value):
+        '''returns True if the urlid of the value passed is from an external source'''
+        try:
+            return value.urlid is not None and not value.urlid.startswith(settings.SITE_URL)
+        except:
+            return False
 
-class LDPSource(models.Model):
-    container = models.URLField()
+
+class LDPSource(Model):
     federation = models.CharField(max_length=255)
 
-    class Meta:
-        rdf_type = 'sib:source'
+    class Meta(Model.Meta):
+        rdf_type = 'ldp:Container'
         ordering = ('federation',)
-        permissions = (
-            ('view_source', 'acl:Read'),
-            ('control_source', 'acl:Control'),
-        )
+        container_path = 'sources'
+        lookup_field = 'federation'
 
     def __str__(self):
-        return "{}: {}".format(self.federation, self.container)
+        return "{}: {}".format(self.federation, self.urlid)
+
+
+@receiver([post_save])
+def auto_urlid(sender, instance, **kwargs):
+    if isinstance(instance, Model):
+        if getattr(instance, Model.slug_field(instance), None) is None:
+            setattr(instance, Model.slug_field(instance), instance.pk)
+            instance.save()
+        if (instance.urlid is None or instance.urlid == '' or 'None' in instance.urlid):
+            instance.urlid = instance.get_absolute_url()
+            instance.save()
+
+
+if 'djangoldp_account' not in settings.DJANGOLDP_PACKAGES:
+    def webid(self):
+        # hack : We user webid as username for external user (since it's an uniq identifier too)
+        if validators.url(self.username):
+            webid = self.username
+        # unable to use username, so use user-detail URL with primary key
+        else:
+            webid = '{0}{1}'.format(settings.BASE_URL, reverse_lazy('user-detail', kwargs={'pk': self.pk}))
+        return webid
+
+    get_user_model()._meta.serializer_fields = ['@id']
+    get_user_model().webid = webid
